rem Location of download cache
rem set download_cache_dir=%TEMP%

rem China mirror for downloading Git for Windows installer
rem set git_mirror=taobao

rem Do not update packages; only download missing files.
rem CAUTION: may suffer from incomplete downloads.
rem set no_update=1

rem Location of Rime configuration manager and downloaded packages
rem set plum_dir=%APPDATA%\plum

rem Location of Rime user directory
rem set rime_dir=%APPDATA%\Rime

rem Disable /plum/ bash script; use batch installer only.
rem set use_plum=0
